@@ -157,6 +157,8 @@ NumberofRepeats=1)
 
 #			filtered <- names(unitPvalues);
 			filtered <- correlated_Remove(data,names(unitPvalues),thr=0.99);
+			attr(filtered,"CorrMatrix") <- NULL;
+
 			if (length(filtered) > 1) featureSize <- featureSize*length(filtered)/length(unitPvalues);
 
 			size <- length(filtered);
@@ -582,6 +584,11 @@ NumberofRepeats=1)
 		}
 		bagg <- baggedModel(formula.list,data,type,univariate=unirank,useFreq=FALSE,equifreqCorrection=equiMaxFreq,n_bootstrap=1);
 	}
+	selectedfeatures <- names(firstModel$at.opt.model$coefficients)[-1];
+	if (!is.null(bagg))
+	{
+		selectedfeatures <- names(bagg$frequencyTable);
+	}
 
 
 
@@ -593,7 +600,8 @@ NumberofRepeats=1)
 		formula.list=formula.list,
 		forward.selection.list=forward.selection.list,
 		oridinalModels=oridinalModels,
-		equivalent=equivalent
+		equivalent=equivalent,
+		selectedfeatures = selectedfeatures
 	);
 	class(result) <- c("fitFRESA","BSWiMS");
 
