@@ -18,6 +18,17 @@
  
  
 #include "FRESAcommons.h" 
+#include "Rinternals.h"
+#include "Rversion.h"
+#include <stdlib.h> // for NULL
+#include <R_ext/Rdynload.h>
+
+using namespace R; 
+
+#if R_VERSION < R_Version(4, 2, 0)
+extern "C" void wilcox_free(void);
+#endif
+
  
 #define EPS 1e-4		 
  
@@ -1445,13 +1456,13 @@ improvedRes improvedResidualsFunc(const vec &oldResiduals,const vec &newResidual
     improvedRes result; 
  
  
-	double pwil = 0.5; 
-	double pbin = 0.5; 
-	double ptstu = 0.5; 
-	double f_test = 0.5; 
+	double pwil = 1.0; 
+	double pbin = 1.0; 
+	double ptstu = 1.0; 
+	double f_test = 1.0; 
 	double p1=0.0; 
 	double p2=0.0; 
-	double pvalue = 0.5; 
+	double pvalue = 1.0; 
 	double size = oldResiduals.n_elem; 
 	double improved = 0.0; 
 	if (size==0)  
@@ -1464,8 +1475,8 @@ improvedRes improvedResidualsFunc(const vec &oldResiduals,const vec &newResidual
 	double increase  = 0.0; 
 	for (int i=0; i< size;i++) 
 	{ 
-		reduction += (newres[i]<oldres[i]); 
-		increase += (newres[i]>oldres[i]); 
+		reduction += (newres[i] <= oldres[i]); 
+		increase += (newres[i] > oldres[i]); 
 	} 
 	p1 = reduction/size;															//	#proportion of subjects with improved residuals 
 	p2 = increase/size;											//#proportion of subjects with worst residuals 
@@ -1481,7 +1492,7 @@ improvedRes improvedResidualsFunc(const vec &oldResiduals,const vec &newResidual
 	 
 	if (rss1 > 1000*DOUBLEEPS) 
 	{ 
-		pvalue = 0.5; 
+		pvalue = 0.0; 
 		pwil = pvalue; 
 		pbin =  pvalue; 
 		ptstu =  pvalue; 
@@ -1501,7 +1512,7 @@ improvedRes improvedResidualsFunc(const vec &oldResiduals,const vec &newResidual
 			{ 
 				case 1: 
 				{ 
-					pbin = 0.5; 
+					pbin = 1.0; 
 					if (reduction>=increase) pbin = binomtest(reduction,size,0.5,tail); 
 					pvalue = pbin;	 
 					break; 
@@ -1511,7 +1522,7 @@ improvedRes improvedResidualsFunc(const vec &oldResiduals,const vec &newResidual
 					rss2 = rss1/rss2; 
 //					Rcout << "\n 1:" << rss2 << "\n"; 
 					if (rss2>10000.0) rss2=10000.0; 
-					pvalue = f_test = (1.0-R::pf(size*(rss2-1.0),1.0,size,1,0))*0.5; 
+					pvalue = f_test = 0.5*(1.0-R::pf(size*(rss2-1.0),1.0,size,1,0)); 
 					break; 
 				} 
 				case 3: 
@@ -1530,12 +1541,12 @@ improvedRes improvedResidualsFunc(const vec &oldResiduals,const vec &newResidual
 					rss2 = rss1/rss2; 
 					if (rss2>10000.0) rss2=10000.0; 
 //					Rcout << "\n 1:" << rss2 << "\n"; 
-					f_test = (1.0-R::pf(size*(rss2-1.0),1.0,size,1,0))*0.5; 
+					f_test = 0.5*(1.0-R::pf(size*(rss2-1.0),1.0,size,1,0)); 
 					pvalue =  f_test; 
-					pbin=0.5; 
+					pbin=1.0; 
 					pwil=1.0; 
 					if (reduction>=increase) pbin = binomtest(reduction,size,0.5,tail); 
-					if (reduction>=increase) pwil = wilcoxtest(oldres, newres,0.0,TRUE,tail,TRUE); 
+					if (reduction >= 0.5*increase) pwil = wilcoxtest(oldres, newres,0.0,TRUE,tail,TRUE); 
 //					ptstu = ttest(oldres, newres,0.0,TRUE,TRUE,tail); 
 					ptstu = pttest(oldres, newres,tail); 
 					break; 
@@ -1763,18 +1774,29 @@ double wilcoxtest(const vec &xt,const vec &y , double mu, bool paired,const std:
 		 
         if(exact && !ties)  
         { 
-        	 if(tail=="greater") 
-                pvalue =R::pwilcox(stat - 1, nx, ny, 0,0); 
-             else if(tail=="less") pvalue =R::pwilcox(stat, nx, ny,1,0); 
+        	 if(tail=="greater")
+			 {
+                pvalue = R::pwilcox(stat - 1, nx, ny, 0,0); 
+			 }
+             else if(tail=="less") 
+			 {
+				 pvalue = R::pwilcox(stat, nx, ny,1,0); 
+			 }
              else 
                    { 
                    	double p; 
-                       if(stat > (nx * ny / 2.0)) 
-                           p =R::pwilcox(stat - 1, nx, ny, 0,0); 
-                       else 
-                           p =R::pwilcox(stat, nx, ny,1,0); 
-                        pvalue =std::min(2.0 * p, 1.0); 
-                   } 
+                       if(stat > (nx * ny / 2.0))
+					   {						   
+                           p = R::pwilcox(stat - 1, nx, ny, 0,0); 
+					   }
+                       else
+					   {						   
+                           p = R::pwilcox(stat, nx, ny,1,0);
+					   }
+                        pvalue = std::min(2.0 * p, 1.0); 	
+                   }
+				   
+			wilcox_free();
         } 
         else  
         { 
